﻿EnableExplicit


; Specify proxy server here (or leave empty to not use proxy)
; #PROXY_AGENT = "socks5://127.0.0.1:9024"
#PROXY_AGENT = ""



;{ PCURL base }
	
	; There is one and single imported function. It can be safely used from threads here.
	
	; Downloads given URL into a memory buffer
	; *lp_size			pointer to a variable which receives size of returned buffer. can be null
	; URL$				HTTP/HTTPS
	; Timeout			seconds to wait for server responce
	; Proxy$			proxy server adress		 	(https://curl.haxx.se/libcurl/c/CURLOPT_PROXY.html)
	; Referer$			HTTP referer 				(https://curl.haxx.se/libcurl/c/CURLOPT_REFERER.html)
	; Cookies$			lot of cookies 				(https://curl.haxx.se/libcurl/c/CURLOPT_COOKIE.html)
	; UserAgent$		UA string 					(https://curl.haxx.se/libcurl/c/CURLOPT_USERAGENT.html)
	; ReturnHeaders		if True, responce headers returned in data buffer, else it contains downloaded file/text only
	; RETURN:			always non-zero: pointer to a buffer with received data. no need to release it.
	;					additionaly, that buffer is always null-terminated, so it is safe to read string from it without setting size
	Prototype ReceiveHTTP (*lp_size.Long, URL$, Timeout, Proxy$ = "", Referer$ = "", Cookies$ = "", UserAgent$ = "", ReturnHeaders = #False)
	
	Global hCurl = OpenLibrary(#PB_Any, "PCURL.dll")
	Global ReceiveHTTP.ReceiveHTTP	
	If IsLibrary(hCurl)
		ReceiveHTTP = GetFunction(hCurl, "ReceiveHTTP")
	Else
		ReceiveHTTP = 0
	EndIf
	
	; Download given url and save to specified file
	; RETURN:		true on success (or if file already exists and OverWrite = false)
	Procedure DownloadBin (FileName$, Url$, OverWrite = #False)
		Protected RetryCount = 8
		
		If Url$
			; download
			Protected DownloadedBytes = 0
			; form image path on a disk
			Protected ImgPath$ = FileName$
			If FileSize(ImgPath$) = -1 Or OverWrite = #True
				Repeat
					Protected hData = ReceiveHTTP(@DownloadedBytes, Url$, 90, #PROXY_AGENT, "https://vk.com/", "", "Mozilla/5.0 (Windows NT 6.1; rv:31.0) Gecko/20100101 Firefox/31.0 K-Meleon/75.0", #False)
					If DownloadedBytes
						CreateFile(134, ImgPath$)
						WriteData(134, hData, DownloadedBytes)
						CloseFile(134)
						ProcedureReturn #True
						Break
					Else
						; retry
						RetryCount - 1
						If RetryCount
							Delay(1000)
						Else
							Break
						EndIf
					EndIf
				ForEver
			Else
				; Skipped: file exists
				ProcedureReturn #True
			EndIf
		EndIf
		ProcedureReturn #False
	EndProcedure
	
	
;}

;{ Commmon }
	
	
	;{ HTML TAG CONSTRUCTORS }
		
		Procedure$ HTML_IMG (src$, alt$ = "", w = 0, h = 0)
			Protected res$ = ~"<img src=\"" + src$ + ~"\""
			
			If alt$
				res$ + ~" alt=\"" + alt$ + ~"\""
			EndIf
			If w
				res$ + ~" width=\"" + Str(w) + ~"\""
			EndIf
			If h
				res$ + ~" height=\"" + Str(h) + ~"\""
			EndIf
			
			res$ + ">"
			
			ProcedureReturn res$
		EndProcedure
		
		Procedure$ HTML_A (href$, title$ = "", style$ = "")
			Protected res$ = ~"<a href=\"" + href$ + ~"\" target=\"_blank\""
			If style$ = ""
				style$ = "color:#0000EE"
			EndIf
			res$ + ~" style=\"" + style$ + ~"\""
			
			res$ + ">" + title$ + "</a>"
			ProcedureReturn res$
		EndProcedure
		
		Procedure$ HTML_B (text$, style$ = "")
			Protected res$ = "<b"
			
			If style$ 
				res$ + ~" style=\"" + style$ + ~"\""
			EndIf
			res$ + ">" + text$ + "</b>"
			
			ProcedureReturn res$
		EndProcedure
		
		Procedure$ HTML_I (text$, style$ = "")
			Protected res$ = "<i"
			
			If style$ 
				res$ + ~" style=\"" + style$ + ~"\""
			EndIf
			res$ + ">" + text$ + "</i>"
			
			ProcedureReturn res$
		EndProcedure
		
	;}
	
	
	; Walks through given path and creates directories on the way
	; Path$		full folder path
	; RETURN:	1 on success, 0 if failed to create directory
	Procedure ReCreatePath (Path$)
		Protected a, s$, t$
		Path$ = ReplaceString(Path$, "/", "\")
		Repeat
			a + 1
			t$ = StringField(Path$, a, "\") 
			
			If t$
				s$ + t$ + "\"
				
				If FileSize(s$) = -1 ; doesn't exists
					If Not CreateDirectory(s$)
						ProcedureReturn 0 ; failed to recreate path
					EndIf
				EndIf
			Else
				Break
			EndIf
		Until s$ = ""
		ProcedureReturn 1
	EndProcedure
	
	
;}


;{ VK XML parsing }

	
	;{ VK base shit }
		
		; Performs API requests and returns responce in XML format, using PCURL
		; Method$		VK "api" method, see (https://vk.com/dev/methods) for all the shit
		; Params$		List of request params separated with "&"
		; Token$		Access Token
		; RETURN:		HTTP responce (XML content), or empty string on connection/PCURL error
		Procedure$ VkRequest (Method$, Params$, Token$ = "")
			; anyway add a protection to not catch some error
			Static LO 
			If LO > ElapsedMilliseconds()
				Delay(LO - ElapsedMilliseconds())
			EndIf
			LO = ElapsedMilliseconds() + 333
			
			Protected APIBase$ = "https://api.vk.com/method/"
			Protected URL$ = APIBase$ + Method$ + ".xml?" + Params$
			If Token$
				URL$ + "&access_token=" + Token$
			EndIf
			
			If ReceiveHTTP
				ProcedureReturn PeekS(ReceiveHTTP (0, URL$, 60, #PROXY_AGENT, "", "", "", #False), -1, #PB_UTF8)
			EndIf
		EndProcedure
		
		
		
	;}
	
	
	;{ VK users }
		
		; /response/user
		Structure XML_USER
			bdate.s
			city.q
			country.q
			domain.s
			faculty.q
			faculty_name.s
			first_name.s
			graduation.q
			home_phone.s
			home_town.s
			last_name.s
			mobile_phone.s
			photo_max_orig.s
			sex.q
			site.s
			status.s
			uid.q
			university.q
			university_name.s
		EndStructure	
		
		; Returns string with full user name
		; uid		user ID
		Procedure$ VkGetUserName (uid, Token$)
			Protected XML$ = VkRequest("users.get", "user_ids=" + Str(uid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			; Temp
			Protected tUser.XML_USER
			Protected nodeComment
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/user[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_USER)	
							Res$ = tUser\first_name + " " + tUser\last_name
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res$
		EndProcedure
		
		; Returns list of all user friends, visible with specified token
		; uid		user ID
		Procedure VkGetUserFriends(List Out.XML_USER(), uid, Token$)
			Protected XML$ = VkRequest("friends.get", "order=name&count=9999&fields=uid,first_name,last_name&user_id=" + Str(uid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeComment
			
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						; search for nodes of type "post"
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/user[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
							
							AddElement(Out())					
							ExtractXMLStructure(nodeComment, Out(), XML_USER)
						ForEver
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
		EndProcedure
		
		; Returns list of all user followers
		; uid		user ID
		Procedure VkGetUserFollowers(List Out.XML_USER(), uid, Token$)
			Protected XML$ = VkRequest("users.getFollowers", "order=name&count=9999&fields=uid,first_name,last_name&user_id=" + Str(uid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeComment
			
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/items/user[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
							
							AddElement(Out())					
							ExtractXMLStructure(nodeComment, Out(), XML_USER)
						ForEver
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
		EndProcedure
		
		; Returns HTML code containing some user profile info
		Procedure$ VkGetUserInfoHTML (uid, Token$)
			Protected XML$ = VkRequest("users.get", "fields=sex,bdate,city,country,home_town,photo_max_orig,domain,contacts,site,education,status,occupation&user_ids=" + Str(uid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected nodeComment
			; Temp
			Protected tUser.XML_USER
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/user[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_USER)
							
							Res$ + HTML_A("https://vk.com/" + tUser\domain, HTML_B(tUser\first_name + " " + tUser\last_name, "color:#B94426")) + "<br>"
							Res$ + "'" + tUser\status + "'" + "<br>"
							
							
							Res$ + "<br>"
							If tUser\site
								Res$ + "Site: " + tUser\site + "<br>"
							EndIf
							If tUser\bdate
								Res$ + "Birthday: " + tUser\bdate + "<br>"
							EndIf	
							Select tUser\sex
								Case 1: Res$ + "(.)(.)<br>"
								Case 2: Res$ + "8====<br>"
							EndSelect							
							
							If tUser\home_phone
								Res$ + "Phone: " + tUser\home_phone + "<br>"
							EndIf
							If tUser\mobile_phone
								Res$ + "Mobile: " + tUser\mobile_phone + "<br>"
							EndIf
							If tUser\home_town
								Res$ + "Home: " + tUser\home_town + "<br>"
							EndIf
							If tUser\city
								Res$ + "City: " + tUser\city + "<br>"
							EndIf							
							If tUser\university_name
								Res$ + "University: " + tUser\university_name + "<br>"
							EndIf									
							If tUser\faculty_name
								Res$ + "Faculty: " + tUser\faculty_name + "<br>"
							EndIf							
							
							; attach friends list
							Protected NewList Friends.XML_USER()
							VkGetUserFriends(Friends(), uid, Token$)
							Protected Friends$
							ForEach Friends()
								
								Friends$ + HTML_A("https://vk.com/id" + Friends()\uid, Friends()\first_name + " " + Friends()\last_name, "color:#006BB1") + "<br>"
							Next
							If Friends$
								Res$ + "<br><br>" + HTML_B("[Friends]", "color:#8F8F8F") + "<br>" + Friends$
							EndIf
							; and followers list
							Protected NewList Followers.XML_USER()
							Protected Followers$
							VkGetUserFollowers(Followers(), uid, Token$)
							ForEach Followers()
								Followers$ + HTML_A("https://vk.com/id" + Followers()\uid, Followers()\first_name + " " + Followers()\last_name, "color:#006BB1") + "<br>"
							Next
							If Followers$
								Res$ + "<br><br>" + HTML_B("[Followers]", "color:#8F8F8F") + "<br>" + Followers$
							EndIf
							Res$ + "<br><br>"
							Res$ + "Backup date: " + FormatDate("%yyyy/%mm/%dd %hh:%ii:%ss", Date()) + "<br>"
							Res$ + "<br><br>"
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res$
		EndProcedure
		
		
		; Download and save user profile photo
		Procedure VkSaveUserPhoto (Filename$, uid, Token$)
			Protected XML$ = VkRequest("users.get", "fields=photo_max_orig&user_ids=" + Str(uid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected nodeComment
			; Temp
			Protected tUser.XML_USER
			Protected Res
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; main node of XML api responce
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/user[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_USER)
							Res = DownloadBin(Filename$, tUser\photo_max_orig, #True)
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res
		EndProcedure
		
	;}
	
	;{ VK groups }
		
		Structure XML_GROUP_COUNTERS
			topics.q
			videos.q
			docs.q
			photos.q
			albums.q
			audios.q
		EndStructure
		Structure XML_GROUP
			admin_level.q
			counters.XML_GROUP_COUNTERS
			description.s
			gid.q
			is_admin.q
			is_closed.q
			is_member.q
			members_count.q
			name.s
			photo.s
			photo_big.s
			photo_medium.s
			screen_name.s
			type.s
		EndStructure
		
		; Returns group name
		; gid 		group ID, MUST BE POSITIVE though VK uses "-" prefix for those IDs
		Procedure$ VkGetGroupName (gid, Token$)
			Protected XML$ = VkRequest("groups.getById", "group_id=" + Str(gid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected nodeComment
			; Temp
			Protected tUser.XML_GROUP
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; main node of XML api responce
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/group[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_GROUP)	
							Res$ = tUser\name
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res$
		EndProcedure
		
		; Returns HTML-formatted group info
		; gid 		group ID, MUST BE POSITIVE though VK uses "-" prefix for those IDs
		Procedure$ VkGetGroupInfoHTML (gid, Token$)
			Protected XML$ = VkRequest("groups.getById", "fields=description,members_count,counters&group_ids=" + Str(gid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected nodeComment
			; Temp
			Protected tUser.XML_GROUP
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; main node of XML api responce
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/group[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_GROUP)	
							
							Res$ + HTML_A("https://vk.com/" + tUser\screen_name, HTML_B(tUser\name, "color:#B94426")) + "<br><br>"
							Res$ + Str(tUser\members_count) + " members" + "<br>"
							Res$ + "Type: " + tUser\type + "<br><br>"
							Res$ + tUser\description + "<br><br>"
							Res$ + "Backup date: " + FormatDate("%yyyy/%mm/%dd %hh:%ii:%ss", Date()) + "<br>"
							Res$ + "<br><br>"
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res$
		EndProcedure
		
		; Download and save group avatar
		; gid 		group ID, MUST BE POSITIVE though VK uses "-" prefix for those IDs
		Procedure VkSaveGroupPhoto (Filename$, gid, Token$)
			Protected XML$ = VkRequest("groups.getById", "fields=photo_big&group_ids=" + Str(gid), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected nodeComment
			; Temp
			Protected tUser.XML_GROUP
			Protected Res
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/group[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, tUser, XML_GROUP)
							Protected PhotoUrl$
							
							If tUser\photo_big
								PhotoUrl$ = tUser\photo_big
							ElseIf tUser\photo_medium
								PhotoUrl$ = tUser\photo_medium
							ElseIf tUser\photo
								PhotoUrl$ = tUser\photo
							EndIf
							Res = DownloadBin(Filename$, PhotoUrl$, #True)
						EndIf
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res
		EndProcedure
		
	;}
	
	;{ VK user+groups common }
		
		; Unified ID resolver with internal cache system 
		; id		user or group/community ID
		; RETURN:	string with user/community full name
		Procedure$ VkResolveID (id, Token$)
			; cached Names to avoid additional queries
			Static NewMap CachedNames$()
			
			Protected res$
			Protected retry = 3
			
			; resolve only if missing from cache
			If FindMapElement(CachedNames$(), Str(id)) = 0
				While res$ = "" And retry > 0
					If id < 0
						; this is community
						res$ = VkGetGroupName(Abs(id), Token$)
					Else
						res$ = VkGetUserName(id, Token$)
					EndIf
					
					retry - 1
				Wend
				If res$
					CachedNames$(Str(id)) = res$
				EndIf
			Else
				res$ = CachedNames$(Str(id))
			EndIf
			
			ProcedureReturn res$
		EndProcedure
		
	;}
	
	
	
	;{ VK posts and attachments }
		
		; Anything related with wall posts (also used to parse IM attachments)
		; Structures may be incomplete 
		
		;{ Post attachments }
		
			Structure XML_AUDIO
				aid.q
				owner_id.q
				artist.s
				title.s
				duration.q
				url.s
				content_restricted.q
				performer.s
				lyrics_id.q
				genre.q
			EndStructure
			Structure XML_PHOTO
				access_key.s
				aid.q
				created.q
				height.q
				owner_id.q
				pid.q
				post_id.q
				src.s
				src_big.s
				src_small.s
				src_xbig.s
				src_xxbig.s
				src_xxxbig.s
				text.s
				user_id.q
				width.q
			EndStructure
			Structure XML_LINK
				description.s
				image_big.s
				image_src.s
				preview_page.s
				preview_url.s
				target.s
				title.s
				url.s
			EndStructure
			Structure XML_DOC
				did.q
				owner_id.q
				title.s
				size.q
				ext.s
				url.s
				date.q
				thumb.s
				thumb_s.s
				access_key.s
			EndStructure
			Structure XML_VIDEO
				access_key.s
				can_edit.q
				date.q
				description.s
				duration.q
				image.s
				image_big.s
				image_small.s
				image_xbig.s
				owner_id.q
				platform.s
				title.s
				vid.q
				views.q
			EndStructure
			Structure XML_POLL
				poll_id.q
				question.s
			EndStructure
			Structure XML_WALL
				can_delete.q
				date.q
				from_id.q
				id.q
				post_type.s
				text.s
				to_id.q
			EndStructure
			Structure XML_WALL_REPLY
				cid.q
				date.q
				from_id.q
				owner_id.q
				post_id.q
				text.s
				uid.q
			EndStructure
			
			Structure XML_ATTACHMENT
				type.s				; type of attachment
				photo.XML_PHOTO		; use if type = "photo"
				audio.XML_AUDIO		; use if type = "audio"
				link.XML_LINK		; use if type = "link"
				doc.XML_DOC			; use if type = "doc"
				video.XML_VIDEO
				poll.XML_POLL		; 
				wall.XML_WALL		; I've encountered this in old IM history
				wall_reply.XML_WALL_REPLY
			EndStructure

		;}

		;{ Post }
		
			Structure XML_MEDIA
				item_id.q
				owner_id.q
				share_title.s
				share_url.s
				thumb_src.s
				type.s
			EndStructure
			
			Structure XML_REPOSTS
				count.q
				user_reposted.q
			EndStructure
			Structure XML_LIKES
				can_like.q
				can_publish.q
				count.q
				user_likes.q
			EndStructure
			Structure XML_COMMENTS
				can_post.q
				count.q
			EndStructure
			Structure XML_POST_SOURCE
				Data.s
				type.s
			EndStructure
			
			; describes "post" node of XML responce
			Structure XML_POST
				can_delete.q
				can_pin.q
				comments.XML_COMMENTS
				copy_comment_id.q
				copy_commenter_id.q
				copy_owner_id.q
				copy_post_date.q
				copy_post_id.q
				copy_post_type.s
				copy_text.s
				date.q
				from_id.q
				id.q
				likes.XML_LIKES
				marked_as_ads.q
				media.XML_MEDIA
				online.q
				post_source.XML_POST_SOURCE
				post_type.s
				reply_count.q
				reposts.XML_REPOSTS
				signer_id.q
				text.s
				to_id.q
			EndStructure
		
		;}
		
		
		
		
		; Finally custom structure to store results (not from VK api)
		Structure XML_POSTUNITED
			post.XML_POST					; post data
			List content.XML_ATTACHMENT()	; list of attachments for this post
		EndStructure
		

		
		; Parses "attachments" node inside of "post" of XML responce
		; Node		XML node to process
		; Out		list to return data
		; RETURN:	true on success
		Procedure VkXMLPostAttachments (Node, List Out.XML_ATTACHMENT())
			Protected tmp, tmp2, nodeAttach, nodeAttachItem
			
			; search for node "attachments" inside of "post" node
			ClearList(Out())
			If Node
				For tmp = 1 To XMLChildCount (Node) 
					nodeAttach = ChildXMLNode (Node, tmp)
					
					; node is "attachments"
					If nodeAttach And GetXMLNodeName (nodeAttach) = "attachments" 
						; extract every "attachment" data inside "attachments"
						For tmp2 = 1 To XMLChildCount (nodeAttach)
							nodeAttachItem = ChildXMLNode (nodeAttach, tmp2)
							
							; extract "attachment" to a new list element
							If nodeAttachItem And GetXMLNodeName (nodeAttachItem) = "attachment" 
								AddElement(Out())
								ExtractXMLStructure(nodeAttachItem, @Out(), XML_ATTACHMENT, #PB_XML_NoCase)
							EndIf
						Next tmp2
						
						; quit to new "post" search
						ProcedureReturn #True
					EndIf
				Next tmp
			EndIf
		EndProcedure
		
		
		; Gets posts data from given VK page
		; Posts()	list of XML_POSTUNITED type to receive results
		; Page$		VK page address (relative, like "asshole_1")
		; Token$	API token
		; Offset	wall post index to start from
		; Count 	amount of posts to get
		; RETURN:	none, wall posts parsed and placed to a Posts() list
		Procedure VkGetWallPosts (List Posts.XML_POSTUNITED(), Page$, Token$, Offset, Count, NoReposts)
			; Query wall data from VK API
			Protected XML$ = VkRequest("wall.get", "domain=" + Page$ + "&offset=" + Str(Offset) + "&filter=all&count=" + Str(Count), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodePost
			; Results
			ClearList(Posts())
			
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; search for nodes of type "post"
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						Repeat
							tmp + 1
							nodePost = XMLNodeFromPath(RootXMLNode (hXML), "/response/post[" + Str(tmp) + "]" )
							If nodePost = 0
								Break
							EndIf
							
							; extract post data
							AddElement(Posts())
							ExtractXMLStructure(nodePost, Posts()\post, XML_POST, #PB_XML_NoCase)
							
							If NoReposts And Posts()\post\post_type = "copy"
								DeleteElement(Posts())
							Else
								; extract all attachments data
								VkXMLPostAttachments(nodePost, Posts()\content())
							EndIf
							
						ForEver
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
		EndProcedure
		
		
		; Returns comments of specified post on given wall
		; RETURN:	formatted HTML text
		Procedure$ VkGetWallCommentsHTML (wall_id, post_id, Token$)
			; /response/comment[]
			Structure XML_COMMENT
				cid.q
				uid.q
				date.q
				from_id.q
				text.s
			EndStructure
			
			Protected XML$ = VkRequest("wall.getComments", "owner_id=" + Str(wall_id) + "&preview_length=0&fields=first_name&count=100&post_id=" + Str(post_id), Token$)
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeComment
			; Temp
			Protected tComment.XML_COMMENT
			
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						; search for nodes of type "post"
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/comment[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
							
							ExtractXMLStructure(nodeComment, tComment, XML_COMMENT)
							
							Protected FromName$
							FromName$ = VkResolveID(tComment\from_id, Token$)
							
							Res$ + HTML_A("https://vk.com/id" + tComment\uid, FromName$, "color:#006BB1") + " " + HTML_I(FormatDate("%yyyy/%mm/%dd %hh-%ii-%ss", tComment\date), "color:#8F8F8F") + "<br>"
							Res$ + tComment\text + "<br><br>"
							
							
						ForEver
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			
			ProcedureReturn Res$
		EndProcedure
		
		; Returns content of poll attached to a wall post
		; RETURN:	formatted HTML text
		Procedure$ VkGetWallPollHTML (wall_id, poll_id, Token$)
			Protected XML$ = VkRequest("polls.getById", "owner_id=" + Str(wall_id) + "&poll_id=" + Str(poll_id), Token$)
			
			; /response/answers/answer[]
			Structure XML_ANSWER
				id.q
				rate.f
				text.s
				votes.q
			EndStructure
			
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeComment
			; Temp
			Protected tComment.XML_ANSWER
			
			
			Protected Res$
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						; search for nodes of type "post"
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/answers/answer[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
							
							ExtractXMLStructure(nodeComment, tComment, XML_ANSWER)
							
							Res$ + Str(tmp) + ". " + tComment\text + " (" + StrF(tComment\rate, 2) + "%), " + Str(tComment\votes) + "<br>"	
						ForEver
					EndIf
				EndIf
				
				FreeXML(hXML)
			EndIf
			ProcedureReturn Res$
			
		EndProcedure
		
		
	;}
	
	;{ VK IM/dialogs }
	
		; /response/message[]
		Structure XML_MESSAGE
			body.s
			date.q
			emoji.q
			mid.q
			out.q
			read_state.q
			title.s
			uid.q
			
		EndStructure
		
		Structure XML_MESSAGE_UNITED
			message.XML_MESSAGE
			List content.XML_ATTACHMENT()
		EndStructure
		
 
		Procedure VkGetDialogsList_query (List Out.XML_MESSAGE_UNITED(), Offset, Count, Token$)
			Protected XML$ = VkRequest("messages.getDialogs", "count=" + Str(count) + "&offset=" + Str(Offset), Token$)
		
			; XML-related handles
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeMain, nodeComment
			ClearList(Out())
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; main node of XML api responce
					nodeMain = XMLNodeFromPath(RootXMLNode (hXML), "/response")
					
					If nodeMain
						; search for nodes of type "post"
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/message[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
		
							AddElement(Out())					
							ExtractXMLStructure(nodeComment, Out()\message, XML_MESSAGE)
						ForEver
					EndIf
				EndIf
		
				FreeXML(hXML)
			EndIf
		EndProcedure
		Procedure VkGetDialogsList (List Out.XML_MESSAGE_UNITED(), Count, Token$)
			Protected NewList TDialogs.XML_MESSAGE_UNITED()
			Protected Offset, Load
			PrintN("Loading dialogs list (" + Str(Count) + " max):" )
			ClearList(Out())
			Repeat
				Load = 200
				If Load + Offset > Count
					Load = Count - Offset
				EndIf
				If (Offset > Count) Or Load = 0
					Break
				EndIf
				PrintN( "+" + Str(Load))
				
				VkGetDialogsList_query(TDialogs(), Offset, Load, Token$)	
				
				If ListSize(TDialogs()) = 0
					Break
				Else
					MergeLists(TDialogs(), Out())
					Offset + 200
				EndIf
			ForEver
			FreeList(TDialogs())
		EndProcedure
		
		; Returns message ID of last message in conversation with given user
		; uid		user or community ID
		Procedure VkGetDialogLastMsg (uid, Token$)
			Protected XML$ = VkRequest("messages.getHistory", "user_id=" + Str(uid) + "&offset=0&rev=0&count=1", Token$)
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected T.XML_MESSAGE
			Protected nodeMain, nodeComment
		
			If IsXML(hXML)
				If RootXMLNode (hXML)
					If XMLNodeFromPath(RootXMLNode (hXML), "/response")
						; search for nodes of type "post"
						nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/message[1]" )
						If nodeComment
							ExtractXMLStructure(nodeComment, T, XML_MESSAGE)
						EndIf
					EndIf
				EndIf
		
				FreeXML(hXML)
			EndIf
		
			ProcedureReturn T\mid
		EndProcedure
		
		
		; Parses "attachments" node inside of "post" of XML responce
		; Node		XML node to process
		; Out		list to return data
		; RETURN:	true on success
		Procedure VkXMLIMAttachments (Node, List Out.XML_ATTACHMENT())
			Protected tmp, tmp2, nodeAttach, nodeAttachItem
		
			; search for node "attachments" inside of "post" node
			ClearList(Out())
			If Node
				For tmp = 1 To XMLChildCount (Node) 
					nodeAttach = ChildXMLNode (Node, tmp)
					
					; node is "attachments"
					If nodeAttach And GetXMLNodeName (nodeAttach) = "attachments" 
						; extract every "attachment" data inside "attachments"
						For tmp2 = 1 To XMLChildCount (nodeAttach)
							nodeAttachItem = ChildXMLNode (nodeAttach, tmp2)
							
							; extract "attachment" to a new list element
							If nodeAttachItem And GetXMLNodeName (nodeAttachItem) = "attachment" 
								AddElement(Out())
								ExtractXMLStructure(nodeAttachItem, @Out(), XML_ATTACHMENT, #PB_XML_NoCase)
							EndIf
						Next tmp2
						
						; quit to new "post" search
						ProcedureReturn #True
					EndIf
				Next tmp
			EndIf
		EndProcedure
		
		Procedure VkGetIMHistory_query (List Out.XML_MESSAGE_UNITED(), uid, Token$, Offset, Count)
			Protected XML$ = VkRequest("messages.getHistory", "user_id=" + Str(uid) + "&offset=" + Str(Offset) + "&rev=1&count=" + Str(Count), Token$)
			Protected hXML = ParseXML(#PB_Any, XML$)
			Protected tmp
			Protected nodeMain, nodeComment
			ClearList(Out())
			
			If IsXML(hXML)
				If RootXMLNode (hXML)
					; main node of XML api responce
					nodeMain = XMLNodeFromPath(RootXMLNode (hXML), "/response")
					
					If nodeMain
						; search for nodes of type "post"
						Repeat
							tmp + 1
							nodeComment = XMLNodeFromPath(RootXMLNode (hXML), "/response/message[" + Str(tmp) + "]" )
							If nodeComment = 0
								Break
							EndIf
		
							AddElement(Out())					
							ExtractXMLStructure(nodeComment, Out()\message, XML_MESSAGE)
							
							; extract all attachments data
							VkXMLIMAttachments(nodeComment, Out()\content())					
						ForEver
					EndIf
				EndIf
		
				FreeXML(hXML)
			EndIf
		EndProcedure
		Procedure VkGetIMHistory (List Out.XML_MESSAGE_UNITED(), uid, Token$, Count, StartOffset)
			Protected NewList THistory.XML_MESSAGE_UNITED()
			Protected Offset, Load
			ClearList(Out())
			Repeat
				Load = 200
				If Load + Offset > Count
					Load = Count - Offset
				EndIf
				If (Offset > Count) Or Load = 0
					Break
				EndIf
				If Offset And Offset % 1000 = 0
					PrintN(Str(Offset / 1000) + "k")
				EndIf
				VkGetIMHistory_query (THistory(), uid, Token$, StartOffset + Offset, Load)
				
				If ListSize(THistory()) = 0
					Break
				Else
					MergeLists(THistory(), Out())
					Offset + 200
				EndIf
			ForEver
			FreeList(THistory())
		EndProcedure


	;}
	
	
;}



; IDE Options = PureBasic 5.60 (Windows - x86)
; CursorPosition = 4
; Folding = +z--z6888--8x488D9044vf-348000-
; EnableXP
; CPU = 1
; SubSystem = /OPENGL
; CompileSourceDirectory
; EnablePurifier
; EnableBuildCount = 0
; EnableExeConstant
; EnableUnicode